#include <avr/io.h>
#include <util/delay.h>
#include <string.h>

#include "util.h"

void init() {
   	// Initial PWM (using Timer/Counter2)
   	TCCR2A |= (1 << COM2A1) | (1 << WGM21) | (1 << WGM20);	// Fast PWM Mode, Clear on OCRA
   	TCCR2B |= (1 << CS22);									// Used fclk/64 prescaler
   	OCR2A=0;                      // Initial the OC2A (PB3) Out to 0  

   	// Set the Timer/Counter Control Register
   	TCCR1A = (1<<COM1A1)|(1<<COM1A0); // Set OC1A when up counting, clear when down counting
   	TCCR1B = (1<<WGM13);              // Phase/Freq-correct PWM, top value = ICR1
}

void setup_pins(void) {
	DDRD = 0xFF;
   	PORTD = 1<<PD7;

   	// Set port A as input
	DDRA = 0x00;
	PORTA = 0xFF;

	// Set port B as output
	DDRC = 0xFF;
	PORTC = 0x00;
}

void play_note(unsigned int note,int duration)
{
   	// Reset the 16 bit Counter
   	TCNT1H = 0;
   	TCNT1L = 0;

   	// Set the Counter TOP
   	ICR1H = (note >> 8) & 0x00FF;
   	ICR1L = note;

   	// Turn on the Prescaler
   	TCCR1B |= (1<<CS11);
   	switch(duration) {
		case 200:
			_delay_ms(pause1);
			break;
		case 500:
			_delay_ms(pause2);
			break;
		case 1000:
			_delay_ms(pause3);
			break;
		case 1500:
			_delay_ms(pause4);
			break;
        	case 5000:
			_delay_ms(pause5);
			break;
   	}
    
   	// Turn off the Prescaler
   	TCCR1B &= ~(1 << CS11);
   	_delay_ms(PAUSE);
}

void play_twinkle(unsigned char repeat) {
   	int iCount;

	// Display Clear, Home
   	play_note(C6,200);
   	play_note(C6,200);
   	play_note(G6,200);
   	play_note(G6,200);
   	play_note(A6,200);
   	play_note(A6,200);
   	play_note(G6,500);

      
   	play_note(F6,200);
   	play_note(F6,200);
   	play_note(E6,200);
   	play_note(E6,200);
   	play_note(D6,200);
   	play_note(D6,200);

   	if (!repeat) {
    	play_note(C6,500);
	} else {
    	play_note(C6,1000);
    }

   	if (!repeat) {
    	// Display Clear
        play_note(G6,200);
     	play_note(G6,200);
     	play_note(F6,200);
     	play_note(F6,200);
     	play_note(E6,200);
     	play_note(E6,200);
     	play_note(D6,500); 

     	play_note(G6,200);
     	play_note(G6,200);
     	play_note(F6,200);
     	play_note(F6,200);
     	play_note(E6,200);
     	play_note(E6,200);
     	play_note(D6,500);
    }

   	if(repeat) {
	    	_delay_ms(1000);
    	repeat=0;
    	play_note(C7,100); play_note(C7,1500); // Tada
     	_delay_ms(5000);

        // Switch Off the Backlight
     	for (iCount = 255 ; iCount > 0 ; iCount--) {
       		OCR2A=iCount;			    // Decrease OCR2A
       		_delay_ms(3);
     	}
    } else {
    	repeat=1;
    }
}

/*
 * Functia porneste o noua conversie pentru canalul precizat.
 * In modul fara intreruperi, apelul functiei este blocant. Aceasta se
 * intoarce cand conversia este finalizata.
 *
 * In modul cu intreruperi apelul NU este blocant.
 *
 * @return Valoarea numerica raw citita de ADC de pe canlul specificat.
 */
uint16_t ADC_get(uint8_t channel)
{
    // start ADC conversion on "channel"
    // wait for completion
    // return the result
    ADMUX = 0;
    ADMUX = (1<<REFS0) | channel;

    ADCSRA |= (1 << ADSC);
    while(ADCSRA & (1 << ADSC));

    return ADC;
}

int array_exist(char key) {
	int i;
	for (i = 0 ; i < 12 ; i++) {
		if (pressed_array[i] == key) {
			return 1;
		}
		else {
			return 0;
		}
	}
	return 0;
}

void array_del(char key) {
	int i;
	for (i = 0 ; i < 12 ; i++) {
		if (pressed_array[i] == key) {
			pressed_array[i] = 0;
		}
	}
}

void array_add(char key) {
	int i;
	for (i = 0 ; i < 12 ; i++) {
		if (pressed_array[i] == 0) {
			pressed_array[i] = key;
		}
	}
}

int main(void)
{
	ADCSRA = (1<<ADEN)|(1<<ADPS2)|(1<<ADPS1)|(1<<ADPS0);
   	init();
   	setup_pins();

	DDRB = 0x00;
	PORTB = 0xFF;

   	int row, column;

   	for(;;) {

	   	// Get the 8 bit ADC Result
	   	OCR1AH=0;
	   	OCR1AL=64;

	   	if (!(PINB & (1 << PB0))) {
	   		memcpy(sounds, configuration1, sizeof(configuration1));
	   	}
	   	if (!(PINB & (1 << PB1))) {
	   		memcpy(sounds, configuration2, sizeof(configuration2));
	   	}
	   	if (!(PINB & (1 << PB2))) {
	   		memcpy(sounds, configuration3, sizeof(configuration3));
	   	}
	   	if (!(PINB & (1 << PB3))) {
	   		memcpy(sounds, configuration4, sizeof(configuration4));
	   	}
	   	if (!(PINB & (1 << PB4))) {
	   		memcpy(sounds, configuration5, sizeof(configuration5));
	   	}
	   	if (!(PINB & (1 << PB5))) {
	   		memcpy(sounds, configuration6, sizeof(configuration6));
	   	}

	   	for (column = 0 ; column < 8 ; column++) {
			DDRC = 0x01 << column;
			for (row = 0 ; row < 8 ; row++) {
				if (ADC_get(row) < 550) {		// Check if button is pressed
					_delay_ms(10);				// De-bounce
					if (ADC_get(row) < 550) {	// Check, again, if button is pressed
						pressed = keypad[1][row];
						temp_result = array_exist(pressed);   		// Check if button pressed is in array
						if (temp_result == 0) {               		// Button is pressed first time
							array_add(pressed);				  		// Push button into array
							play_note(sounds[row][column], 200);	// Play note
						} 
					}
				} else {
					// we have to check if keypad[1][i] is not pressed (but exist in temp array)
					pressed = keypad[1][row];
					temp_result = array_exist(pressed); // check if key checked is in array
					if (temp_result == 1) {             // then if its true
						array_del(pressed);             // remove key checked from array 
														// and send noteOff to UART
					}
				}
			}
		}
    }

  	return 0;
}